use strict;
use warnings;
use Test::More;

use_ok 'HTTP::API::Client';
use_ok 'HTTP::API::Client::Response';
use_ok 'HTTP::API::Client::Error';
use_ok 'HTTP::API::Client::Pagination';
use_ok 'HTTP::API::Client::RateLimit';

done_testing;
